@@ -3,7 +3,7 @@
  * @brief Libertine app wrapper
  */
 /*
- * Copyright 2015 Canonical Ltd
+ * Copyright 2015-2017 Canonical Ltd
  *
  * Libertine is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 3, as published by the
@@ -30,6 +30,8 @@ class ContainerConfigList;
 class LibertineConfig;
 class ContainerAppsList;
 class ContainerArchivesList;
+class ContainerBindMountsList;
+class ContainerOperationDetails;
 
 
 class Libertine
@@ -56,6 +58,8 @@ private:
     ContainerConfigList*            containers_;
     ContainerAppsList*              container_apps_;
     ContainerArchivesList*          container_archives_;
+    ContainerBindMountsList*        container_bind_mounts_;
+    ContainerOperationDetails*      container_operation_details_;
     QQuickView                      view_;
 };
 
