@@ -6,7 +6,7 @@
    :synopsis: A sandbox for running DEB-packaged X11-based applications
 """
 
-# Copyright 2015 Canonical Ltd.
+# Copyright 2015-2017 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -22,21 +22,12 @@
 
 __all__ = [
           # from Libertine
-          'LibertineApplication',
+          'ContainerRunning',
           'LibertineContainer',
-          'LibertineSessionBridge',
-          'LibertineSessionBridge',
-          'HostSessionSocketPair',
-          'SessionSocket',
-          'Socket',
+          'NoContainer',
           'utils',
           ]
 
 __docformat__ = "restructuredtext en"
 
-from libertine.Libertine import LibertineApplication
-from libertine.Libertine import LibertineContainer
-from libertine.Libertine import LibertineSessionBridge
-from libertine.Libertine import HostSessionSocketPair
-from libertine.Libertine import SessionSocket
-from libertine.Libertine import Socket
+from libertine.Libertine import ContainerRunning, LibertineContainer, NoContainer
